@@ -111,11 +111,11 @@ environment variables:
 
 .. envvar:: HTTP_PROXY
 
-   HTTP proxy, in the form 'hostname:port' (e.g. '10.10.1.10:3128')
+   HTTP proxy, in the form 'protocol://hostname:port' (e.g. 'http://10.10.1.10:3128')
 
 .. envvar:: HTTPS_PROXY
 
-   HTTPS proxy, in the form 'hostname:port'
+   HTTPS proxy, in the form 'protocol://hostname:port'
 
 '''
 
@@ -131,12 +131,12 @@ environment variables:
 #     pass
 
 import os, json, time, logging, httplib
-
-import requests
-from requests.exceptions import ConnectionError, HTTPError, Timeout
-from requests.auth import AuthBase
+from .packages import requests
+from .packages.requests.exceptions import ConnectionError, HTTPError, Timeout
+from .packages.requests.auth import AuthBase
 
 logger = logging.getLogger(__name__)
+logging.getLogger('dxpy.packages.requests.packages.urllib3.connectionpool').setLevel(logging.ERROR)
 
 from dxpy.exceptions import *
 from dxpy.toolkit_version import version as TOOLKIT_VERSION
@@ -230,6 +230,12 @@ def DXHTTPRequest(resource, data, method='POST', headers={}, auth=True, timeout=
         auth = AUTH_HELPER
     if config is None:
         config = {}
+
+    # When *data* is bytes but *headers* contains Unicode strings, httplib tries to concatenate them and decode *data*,
+    # which should not be done. Also, per HTTP/1.1 headers must be encoded with MIME, but we'll disregard that here, and
+    # just encode them with the Python default (ascii) and fail for any non-ascii content.
+    headers = {k.encode(): v.encode() for k, v in headers.iteritems()}
+
     # This will make the total number of retries MAX_RETRIES^2 for some errors. TODO: check how to better integrate with requests retry logic.
     # config.setdefault('max_retries', MAX_RETRIES)
     if 'Content-Type' not in headers and method == 'POST':
@@ -264,7 +270,7 @@ def DXHTTPRequest(resource, data, method='POST', headers={}, auth=True, timeout=
             response = SESSION_HANDLER.request(method, url, data=data, headers=headers, timeout=timeout,
                                                auth=auth, config=config, **kwargs)
 
-            if _UPGRADE_NOTIFY and response.headers.get('x-upgrade-info', '').startswith('A recommended update is available') and not os.environ.has_key('ARGPARSE_AUTO_COMPLETE'):
+            if _UPGRADE_NOTIFY and response.headers.get('x-upgrade-info', '').startswith('A recommended update is available') and not os.environ.has_key('_ARGCOMPLETE'):
                 logger.info(response.headers['x-upgrade-info'])
                 try:
                     with file(_UPGRADE_NOTIFY, 'a'):
@@ -472,6 +478,7 @@ def _initialize(suppress_warning=False):
     else:
         if "DX_PROJECT_CONTEXT_ID" in os.environ:
             set_workspace_id(os.environ["DX_PROJECT_CONTEXT_ID"])
+            set_project_context(os.environ["DX_PROJECT_CONTEXT_ID"])
 
 _initialize()
 
