@@ -44,7 +44,7 @@ parser.add_argument("src_dir", help="App or applet source directory (default: cu
 parser.set_defaults(mode="app")
 parser.add_argument("--create-app", help=argparse.SUPPRESS, action="store_const", dest="mode", const="app")
 parser.add_argument("--create-applet", help=argparse.SUPPRESS, action="store_const", dest="mode", const="applet")
-parser.add_argument("-d", "--destination", help="Specifies the destination project, destination folder, and/or name for the applet, in the form [PROJECT_NAME_OR_ID:][/[FOLDER/][NAME]]. Overrides the project, folder, and name fields of the dxapp.json, if they were supplied.", default=None)
+parser.add_argument("-d", "--destination", help="Specifies the destination project, destination folder, and/or name for the applet, in the form [PROJECT_NAME_OR_ID:][/[FOLDER/][NAME]]. Overrides the project, folder, and name fields of the dxapp.json, if they were supplied.", default='.')
 
 parser.set_defaults(use_temp_build_project=True)
 parser.add_argument("--no-temp-build-project", help="When building an app, build its applet in the current project instead of a temporary project", action="store_false", dest="use_temp_build_project")
@@ -360,9 +360,33 @@ def _build_app_remote(mode, src_dir, publish=False, destination_override=None,
 
     build_options = {'dx_toolkit_autodep': dx_toolkit_autodep}
 
-    # These flags are basically passed through verbatim.
     if version_override:
         build_options['version_override'] = version_override
+    elif do_version_autonumbering:
+        # If autonumbering is DISABLED, the interior run of dx-build-app
+        # will detect the correct version to use without our help. If it
+        # is ENABLED, the version suffix might depend on the state of
+        # the git repository. Since we'll remove the .git directory
+        # before uploading, we need to determine the correct version to
+        # use here and pass it in to the interior run of dx-build-app.
+        if do_version_autonumbering:
+            app_spec = _parse_app_spec(src_dir)
+            original_version = app_spec['version']
+            app_describe = None
+            try:
+                app_describe = dxpy.api.app_describe("app-" + app_spec["name"], alias=original_version, always_retry=False)
+            except dxpy.exceptions.DXAPIError as e:
+                if e.name == 'ResourceNotFound':
+                    pass
+                else:
+                    raise e
+            if app_describe is not None:
+                if app_describe.has_key('published') or not do_try_update:
+                    # The version we wanted was taken; fall back to the
+                    # autogenerated version number.
+                    build_options['version_override'] = original_version + get_version_suffix(src_dir)
+
+    # The following flags are basically passed through verbatim.
     if bill_to_override:
         build_options['bill_to_override'] = bill_to_override
     if not do_version_autonumbering:
@@ -408,11 +432,7 @@ def _build_app_remote(mode, src_dir, publish=False, destination_override=None,
         # is huge (e.g. the target directory already has a whole bunch
         # of binaries in it) and interrupt before uploading begins.
         app_tarball_file = os.path.join(temp_dir, "app_tarball.tar.gz")
-        # TODO: figure out if we can use --exclude-vcs here (conditional
-        # on presence of GNU tar). This might require propagating the
-        # --version directly to the interior dx-build-app since in
-        # general that can depend on the git metadata.
-        tar_subprocess = subprocess.Popen(["tar", "-czf", "-", "."], cwd=src_dir, stdout=subprocess.PIPE)
+        tar_subprocess = subprocess.Popen(["tar", "-czf", "-", ".", "--exclude", "./.git"], cwd=src_dir, stdout=subprocess.PIPE)
         with open(app_tarball_file, 'w') as tar_output_file:
             total_num_bytes = 0
             last_console_update = 0
@@ -475,7 +495,7 @@ def _build_app_remote(mode, src_dir, publish=False, destination_override=None,
                 dxpy.DXProject(build_project_id).remove_objects([remote_file.get_id()])
     finally:
         if using_temp_project_for_remote_build:
-            dxpy.api.project_destroy(build_project_id)
+            dxpy.api.project_destroy(build_project_id, {"terminateJobs": True})
         shutil.rmtree(temp_dir)
 
     return
@@ -540,7 +560,7 @@ def build_and_upload_locally(src_dir, mode, overwrite=False, publish=False, dest
                 objects_to_delete = [dxpy.get_dxlink_ids(bundled_resource_obj['id'])[0] for bundled_resource_obj in bundled_resources]
                 if objects_to_delete:
                     dxpy.api.project_remove_objects(dxpy.app_builder.get_destination_project(src_dir, project=working_project),
-                                                  input_params={"objects": objects_to_delete})
+                                                    input_params={"objects": objects_to_delete})
             raise
 
         if dry_run:
@@ -602,7 +622,7 @@ def main(**kwargs):
     if args.src_dir is None:
         args.src_dir = os.getcwd()
 
-    if args.mode == "app" and args.destination:
+    if args.mode == "app" and args.destination != '.':
         parser.error("--destination cannot be used when creating an app (only an applet)")
 
     if not args.remote:
