@@ -17,9 +17,9 @@
 # Miscellaneous utility classes and functions for the dx-app-wizard
 # command-line tool
 
-from dxpy.utils.printing import *
-import os, shutil, subprocess, re
-import json
+import os, shutil, subprocess, re, json
+
+from dxpy.utils.printing import (BOLD, DNANEXUS_LOGO, ENDC, fill)
 
 from . import python
 from . import cpp
@@ -49,7 +49,7 @@ class Completer():
         self.matches = None
         self.choices = choices
 
-    def __call__(self, text, state):
+    def complete(self, text, state):
         if state == 0:
             self.matches = filter(lambda choice: choice.startswith(text),
                                   self.choices)
@@ -63,8 +63,8 @@ def clean(s):
     return "\n".join(line.rstrip() for line in s.split("\n"))
 
 def use_completer(completer=None):
-    if completer_state['available']:
-        readline.set_completer(completer)
+    if completer_state['available'] and completer is not None:
+        readline.set_completer(completer.complete)
 
 # Expect default to be a default string value
 # Expect choices to be a list of strings
