@@ -17,10 +17,30 @@
 #   License for the specific language governing permissions and limitations
 #   under the License.
 
-import os, unittest, json, tempfile, subprocess, csv, shutil, re, base64
+import os, sys, unittest, json, tempfile, subprocess, csv, shutil, re, base64
+import dxpy
 
 from contextlib import contextmanager
 
+@contextmanager
+def chdir(dirname=None):
+    curdir = os.getcwd()
+    try:
+        if dirname is not None:
+            os.chdir(dirname)
+        yield
+    finally:
+        os.chdir(curdir)
+
+class DXCalledProcessError(subprocess.CalledProcessError):
+    def __init__(self, returncode, cmd, output=None, stderr=None):
+        self.returncode = returncode
+        self.cmd = cmd
+        self.output = output
+        self.stderr = stderr
+    def __str__(self):
+        return "Command '%s' returned non-zero exit status %d, stderr:\n%s" % (self.cmd, self.returncode, self.stderr)
+
 def check_output(*popenargs, **kwargs):
     """
     Adapted version of the builtin subprocess.check_output which sets a
@@ -36,17 +56,19 @@ def check_output(*popenargs, **kwargs):
     output, err = process.communicate()
     retcode = process.poll()
     if retcode:
+        print err
         cmd = kwargs.get("args")
         if cmd is None:
             cmd = popenargs[0]
-        exc = subprocess.CalledProcessError(retcode, cmd, output=output)
-        exc.stderr = err
+        exc = DXCalledProcessError(retcode, cmd, output=output, stderr=err)
         raise exc
     return output
 
 def run(command, **kwargs):
     print "$ %s" % (command,)
-    return check_output(command, shell=True, **kwargs)
+    output = check_output(command, shell=True, **kwargs)
+    print output
+    return output
 
 def overrideEnvironment(**kwargs):
     env = os.environ.copy()
@@ -58,18 +80,58 @@ def overrideEnvironment(**kwargs):
             env[key] = kwargs[key]
     return env
 
-class TestDXClient(unittest.TestCase):
-    project = None
+class DXTestCase(unittest.TestCase):
+    # Be sure to use the check_output defined in this module if you wish
+    # to use stderr_regexp. Python's usual subprocess.check_output
+    # doesn't propagate stderr back to us.
+    @contextmanager
+    def assertSubprocessFailure(self, output_regexp=None, stderr_regexp=None, exit_code=3):
+        try:
+            yield
+        except subprocess.CalledProcessError as e:
+            self.assertEqual(exit_code, e.returncode, "Expected command to return code %d but it returned code %d" % (exit_code, e.returncode))
+            if output_regexp:
+                print "stdout:"
+                print e.output
+                self.assertTrue(re.search(output_regexp, e.output), "Expected stdout to match '%s' but it didn't" % (output_regexp,))
+            if stderr_regexp:
+                if not hasattr(e, 'stderr'):
+                    raise Exception('A stderr_regexp was supplied but the CalledProcessError did not return the contents of stderr')
+                print "stderr:"
+                print e.stderr
+                self.assertTrue(re.search(stderr_regexp, e.stderr), "Expected stderr to match '%s' but it didn't" % (stderr_regexp,))
+            return
+        self.assertFalse(True, "Expected command to fail with CalledProcessError but it succeeded")
+
+class TestDXClient(DXTestCase):
+    def setUp(self):
+        proj_name = u"dxclient_test_pröject"
+        self.project = run(u"dx new project '{p}' --brief".format(p=proj_name)).strip()
+        os.environ["DX_PROJECT_CONTEXT_ID"] = self.project
+        # TODO: Fix this once process-wise sessions are in place.  For
+        # now, have to save the old current directory and overwrite
+        # the file.
+        if os.path.exists(os.path.expanduser('~/.dnanexus_config/DX_CLI_WD')):
+            with open(os.path.expanduser('~/.dnanexus_config/DX_CLI_WD')) as fd:
+                self.old_cwd = fd.read()
+            os.remove(os.path.expanduser('~/.dnanexus_config/DX_CLI_WD'))
+        else:
+            self.old_cwd = None
+        if 'DX_CLI_WD' in os.environ:
+            del os.environ['DX_CLI_WD']
 
     def tearDown(self):
         try:
-            run(u"yes|dx rmproject {p}".format(p=TestDXClient.project))
+            run(u"yes|dx rmproject {p}".format(p=self.project))
         except:
             pass
         try:
             os.remove("uploadedfile")
         except:
             pass
+        if self.old_cwd is not None:
+            with open(os.path.expanduser('~/.dnanexus_config/DX_CLI_WD'), 'w') as fd:
+                fd.write(self.old_cwd)
 
     def test_dx_actions(self):
         with self.assertRaises(subprocess.CalledProcessError):
@@ -77,8 +139,6 @@ class TestDXClient(unittest.TestCase):
         run("dx help")
         proj_name = u"dxclient_test_pröject"
         folder_name = u"эксперимент 1"
-        project = run(u"dx new project '{p}' --brief".format(p=proj_name)).strip()
-        os.environ["DX_PROJECT_CONTEXT_ID"] = project
         run("dx cd /")
         run("dx ls")
         run(u"dx mkdir '{f}'".format(f=folder_name))
@@ -176,49 +236,42 @@ class TestDXClient(unittest.TestCase):
         self.assertEqual(desc['size'], second_desc['size'])
         self.assertEqual(desc['length'], second_desc['length'])
 
-        # compare output of old and new
-
-        run(u"yes|dx rmproject {p}".format(p=project))
+    def test_dx_upload_download(self):
+        wd = tempfile.mkdtemp()
+        os.mkdir(os.path.join(wd, "a"))
+        os.mkdir(os.path.join(wd, "a", u"б"))
+        os.mkdir(os.path.join(wd, "a", u"б", "c"))
+        with tempfile.NamedTemporaryFile(dir=os.path.join(wd, "a", u"б")) as fd:
+            fd.write("0123456789ABCDEF"*64)
+            fd.flush()
+            with self.assertSubprocessFailure(stderr_regexp='is a directory but the -r/--recursive option was not given', exit_code=1):
+                run(u'dx upload '+wd)
+            run(u'dx upload -r '+wd)
+            run(u'dx wait "{f}"'.format(f=os.path.join(os.path.basename(wd), "a", u"б", os.path.basename(fd.name))))
+            with self.assertSubprocessFailure(stderr_regexp='is a folder but the -r/--recursive option was not given', exit_code=1):
+                run(u'dx download '+os.path.basename(wd))
+            with chdir(tempfile.mkdtemp()):
+                run(u'dx download -r '+os.path.basename(wd))
+                tree1 = subprocess.check_output("cd {wd}; find .".format(wd=wd), shell=True)
+                tree2 = subprocess.check_output("cd {wd}; find .".format(wd=os.path.basename(wd)), shell=True)
+                self.assertEqual(tree1, tree2)
 
     def test_dx_mkdir(self):
-        project = run(u"dx new project mkdir_test --brief").strip()
-        os.environ["DX_PROJECT_CONTEXT_ID"] = project
-
         with self.assertRaises(subprocess.CalledProcessError):
             run(u'dx mkdir mkdirtest/b/c')
         run(u'dx mkdir -p mkdirtest/b/c')
         run(u'dx mkdir -p mkdirtest/b/c')
         run(u'dx rm -r mkdirtest')
 
-        run(u"dx rmproject --yes {p}".format(p=project))
-
-class TestDXBuildApp(unittest.TestCase):
+class TestDXBuildApp(DXTestCase):
     def setUp(self):
         self.temp_file_path = tempfile.mkdtemp()
+        self.proj_id = dxpy.api.project_new({'name': 'TestDXBuildApp Project'})['id']
+        os.environ['DX_PROJECT_CONTEXT_ID'] = self.proj_id
+
     def tearDown(self):
         shutil.rmtree(self.temp_file_path)
-
-    # Be sure to use the check_output defined in this module if you wish
-    # to use stderr_regexp. Python's usual subprocess.check_output
-    # doesn't propagate stderr back to us.
-    @contextmanager
-    def assertSubprocessFailure(self, output_regexp=None, stderr_regexp=None, exit_code=3):
-        try:
-            yield
-        except subprocess.CalledProcessError as e:
-            self.assertEqual(exit_code, e.returncode, "Expected command to return code %d but it returned code %d" % (exit_code, e.returncode))
-            if output_regexp:
-                print "stdout:"
-                print e.output
-                self.assertTrue(re.search(output_regexp, e.output), "Expected stdout to match '%s' but it didn't" % (output_regexp,))
-            if stderr_regexp:
-                if not hasattr(e, 'stderr'):
-                    raise Exception('A stderr_regexp was supplied but the CalledProcessError did not return the contents of stderr')
-                print "stderr:"
-                print e.stderr
-                self.assertTrue(re.search(stderr_regexp, e.stderr), "Expected stderr to match '%s' but it didn't" % (stderr_regexp,))
-            return
-        self.assertFalse(True, "Expected command to fail with CalledProcessError but it succeeded")
+        dxpy.api.project_destroy(self.proj_id, {'terminateJobs': True})
 
     def write_app_directory(self, app_name, dxapp_str, code_filename=None, code_content="\n"):
         os.mkdir(os.path.join(self.temp_file_path, app_name))
@@ -249,6 +302,8 @@ class TestDXBuildApp(unittest.TestCase):
         self.assertEqual(applet_describe["id"], applet_describe["id"])
         self.assertEqual(applet_describe["name"], "minimal_applet")
 
+    @unittest.skipIf('DXTEST_FULL' not in os.environ,
+                     'skipping test that would create apps')
     def test_build_app(self):
         app_spec = {
             "name": "minimal_app",
@@ -267,6 +322,8 @@ class TestDXBuildApp(unittest.TestCase):
         self.assertEqual(app_describe["name"], "minimal_app")
         self.assertFalse("published" in app_describe)
 
+    @unittest.skipIf('DXTEST_FULL' not in os.environ,
+                     'skipping test that would create apps')
     def test_invalid_project_context(self):
         app_spec = {
             "name": "invalid_project_context",
@@ -325,6 +382,8 @@ class TestDXBuildApp(unittest.TestCase):
         with self.assertSubprocessFailure(exit_code=1):
             run("dx describe " + applet_id)
 
+    @unittest.skipIf('DXTEST_FULL' not in os.environ,
+                     'skipping test that would create apps')
     def test_update_app_categories(self):
         app1_spec = {
             "name": "update_app_categories",
@@ -352,6 +411,8 @@ class TestDXBuildApp(unittest.TestCase):
         run("dx-build-app --json " + app_dir)
         self.assertEquals(json.loads(run("dx api " + app_id + " listCategories"))["categories"], ['B'])
 
+    @unittest.skipIf('DXTEST_FULL' not in os.environ,
+                     'skipping test that would create apps')
     def test_build_app_autonumbering(self):
         app_spec = {
             "name": "build_app_autonumbering",
@@ -406,7 +467,6 @@ class TestDXBuildApp(unittest.TestCase):
         with self.assertSubprocessFailure(stderr_regexp="Entry point file \\S+ has syntax errors"):
             run("dx-build-applet " + app_dir)
 
-
 class TestDXBuildReportHtml(unittest.TestCase):
     def setUp(self):
         self.temp_file_path = tempfile.mkdtemp()
@@ -419,8 +479,13 @@ class TestDXBuildReportHtml(unittest.TestCase):
         html = "<html><body><a href='/'/><a href='/' target='_new'/><img src='img.gif'/><img src='{}'/></body></html>".format(wiki_logo)
         html_file.write(html)
         html_file.close()
+
+        self.proj_id = dxpy.api.project_new({'name': 'TestDXBuildReportHtml Project'})['id']
+        os.environ['DX_PROJECT_CONTEXT_ID'] = self.proj_id
+
     def tearDown(self):
         shutil.rmtree(self.temp_file_path)
+        dxpy.api.project_destroy(self.proj_id, {'terminateJobs': True})
 
     def test_local_file(self):
         run(u"dx-build-report-html {d}/index.html --local {d}/out.html".format(d=self.temp_file_path))
@@ -458,6 +523,7 @@ class TestDXBuildReportHtml(unittest.TestCase):
         self.assertEquals(desc["name"], u"index.html")
         run(u"dx rm {record} {file}".format(record=report["recordId"], file=fileId))
 
-
 if __name__ == '__main__':
+    if 'DXTEST_FULL' not in os.environ:
+        sys.stderr.write('WARNING: env var DXTEST_FULL is not set; tests that create apps will not be run\n')
     unittest.main()
